%{
// This may look like -*- C++ -*- code, but it is really yacc
// 
// $Id: config.y,v 1.12 2000/07/29 22:13:07 voeckler Exp voeckler $
// 
// Author:   Jens-S. Vckler <voeckler@rvs.uni-hannover.de>
// File:     config.y
// 
// (c) 1999 Lehrgebiet Rechnernetze und Verteilte Systeme
//          Universitt Hannover, Germany
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that (i) the above copyright notices and this permission
// notice appear in all copies of the software and related documentation,
// and (ii) the names of the Lehrgebiet Rechnernetze und Verteilte
// Systeme and the University of Hannover may not be used in any
// advertising or publicity relating to the software without the
// specific, prior written permission of Lehrgebiet Rechnernetze und
// Verteilte Systeme and the University of Hannover.
//
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE LEHRGEBIET RECHNERNETZE UND VERTEILTE SYSTEME OR
// THE UNIVERSITY OF HANNOVER BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
// ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
// SOFTWARE.
// 
extern "C" {
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
}

#include "typedefs.h"
#include "global.hh"
#include "symtrie128.hh"
#include "symtrie32.hh"
#include "lltrie.hh"
#include "logline.hh"

// (f)lex compiled with g++, thus must be outside extern "C" ...
MyUInt32 lineno = 1; // make available for flex
int columns;
extern void yyerror( const char* );
extern int yylex();

// some local helper
static MyUInt32 counter; 
static bool insertSubtype( const char* major, LinkedListTrie* minor );
static int  intlog10( MyUInt32 x );
static void prepend( const char* prefix, const char*& sptr );

/* token value union for bison grammar */
struct Item {
  inline Item( const char* s, bool a )
    :string(s),isAlias(a),value(C_U32(0))
    { }
  inline Item( const char* s, MyUInt32 v, bool a )
    :string(s),isAlias(a),value(v)
    { }
  inline Item( const char* s, const char* p, bool a )
    :string(s),isAlias(a),value(p)
    { }
  inline ~Item()
    {
      if ( string ) free((void*) string );
      if ( isAlias && value.alias ) free((void*) value.alias );
    }
  inline void makeTimeout()
    {
      prepend( "TIMEOUT_", string );
      if ( isAlias ) prepend( "TIMEOUT_", value.alias );
      else value.value |= IS_TIMEOUT;
    }
  const char* string;
  bool        isAlias;
  union Combine {
    inline Combine( MyUInt32 v ):value(v) { }
    inline Combine( const char* s ):alias(s) { }

    MyUInt32        value;
    const char*     alias;
  }            value;
};

template <class Trie>
void
insertItem( Trie* trie, Item* item, MyUInt32& cnt )
{
  char msg[2048];
  if ( item->isAlias ) {
    // alias note: The type *IS* *CHECKED* in the main loop!
    MyUInt32 id = trie->find( item->value.alias );
    if ( (id & 0xFFFF) == 0 ) {
      sprintf( msg, "base \"%s\" for alias \"%s\" not defined!", 
	       item->value.alias, item->string );
      yyerror(msg);
    } else if ( (id & IS_ALIAS) == IS_ALIAS ) {
      sprintf( msg, "base \"%s\" for alias \"%s\" is an alias itself!",
	       item->value.alias, item->string );
      yyerror(msg);
    } else {
      if ( trie->insert( item->string, id | IS_ALIAS ) == false ) {
	sprintf( msg, "insertion of \"%s\" failed", item->string );
	yyerror(msg);
      }
    }
  } else {
    // regular item
    if ( trie->insert( item->string,  ++cnt | item->value.value )==false ) {
      sprintf( msg, "insertion of \"%s\" failed", item->string );
      yyerror(msg);
      cnt--;
    }
  }
}
%}
       
%union {
  const char*       lexxer;
  unsigned int     numeric;
  bool             boolean;
  MyUInt32           flags;
  SymbolTrie32*	       map;
  LinkedListTrie*    slist;
  SymbolTrie128*     dlist;
  void*               item; // via void* is less troublesome
}

/* type of terminal symbols */
%token <lexxer> QSTRING ID 
%token <numeric> INTEGER TABLE_SIZE
%token <boolean> BOOLEAN

/* typeless terminal symbols: just token class, no token value */
%token CFG_DEBUG CFG_IDENT CFG_NR_WHITE CFG_IV_PEAK CFG_IV_DAILY
%token CFG_WARN_CRASH CFG_DATARATE CFG_EXT_ICLIENT CFG_IURL_PREFIX
%token CFG_SHOW_DIST CFG_TABLE_SIZE CFG_DNS_CHILDREN CFG_DNS_HELPER
%token CFG_LOG_FQDN CFG_DNS_FILE CFG_DNS_POS_TTL CFG_DNS_NEG_TTL
%token CFG_IRR_FILE CFG_IRR_POS_TTL CFG_IRR_NEG_TTL CFG_IRR_SERVER
%token CFG_IRR_CHILDREN CFG_IRR_HELPER CFG_LENGTH_BYBYTE CFG_REPORT_IV

%token CFG_WARN_METHOD CFG_WARN_HIER CFG_WARN_STATUS CFG_WARN_NONE_TCP
%token CFG_WARN_SCHEME CFG_WARN_SUFFIX
%token CFG_METHOD CFG_HIER CFG_STATUS CFG_MEDIA CFG_SUBTYPES CFG_SCHEME
%token CFG_DOMAIN CFG_SUFFIX CFG_WARN_TLD
%token OPT_ALIAS OPT_TCP OPT_UDP OPT_HIT OPT_MISS OPT_NONE
%token OPT_DIRECT OPT_PEER OPT_PARENT

/* priority ranking of operators: saves additional non-terminal rules! */
/* no operators */

/* type of non-terminals */
%type <map> items hier
%type <slist> slist
%type <dlist> dlist
%type <item> item sitem
%type <flags> option options
%type <flags> stmt

%%
lines   : /* empty rule */
		{ 
		  columns = 0; 
		}
        | lines line
		{ 
		  if ( columns >= 69 ) {
		    fputc( '\n', stderr );
		    columns = 0;
		  }
		}
        ;

line    : stmt ';'
		{ 
#ifdef _REENTRANT
		  flockfile(stderr);
#endif
		  if ( columns == 0 ) {
		    fputs( "# ", stderr );
		    columns = 2;
		  }
		  fprintf( stderr, "..." SF_U32, lineno ); 
		  columns += 3 + intlog10(lineno);
                  if ( $1 > 0 ) {
		    fprintf( stderr, "(" SF_U32 ")", $1 );
		    columns += 2 + intlog10( $1 );
		  }
                  fputc( ' ', stderr );
		  columns++;
#ifdef _REENTRANT
		  funlockfile(stderr);
#endif
		}
        | error ';'
                { 
		  yyerrok; 
		  fprintf( stderr, "reset after ERROR to line " SF_U32 "\n",
			   lineno );
		}
        ;

stmt	: CFG_DEBUG INTEGER
		{ 
		  globals.debugClass = $2;
		  $$ = 0;
		}
	| CFG_DEBUG INTEGER ',' INTEGER
		{
		  globals.debugClass = $2;
		  globals.debugLevel = $4;
		  $$ = 0;
		}
	| CFG_DATARATE BOOLEAN
		{
		  globals.preferDatarate = $2;
		  $$ = 0;
		}
	| CFG_SHOW_DIST BOOLEAN
		{
		  globals.showDistributions = $2;
		  $$ = 0;
		}
	| CFG_EXT_ICLIENT BOOLEAN
		{
		  globals.showExtendedInternal = $2;
		  $$ = 0;
		}
	| CFG_IURL_PREFIX QSTRING
		{
		  if ( globals.iurlPrefix ) 
		    free((void*) globals.iurlPrefix );
		  globals.iurlPrefix = strlen($2) ? strdup($2) : 0;
		  free((void*) $2);
		  $$ = 0;
		}
	| CFG_WARN_CRASH INTEGER
		{       
		  globals.warnCrashInterval = $2;
		  $$ = 0;
		}
	| CFG_IDENT BOOLEAN
		{       
		  globals.noIdent = $2;
		  $$ = 0;
		}
	| CFG_NR_WHITE INTEGER
		{       
		  globals.allowedWhiteSpaces = $2;
		  $$ = 0;
		}
	| CFG_LOG_FQDN BOOLEAN
		{       
		  globals.clientFQDN = $2;
		  $$ = 0;
		}
	| CFG_IV_PEAK INTEGER
		{
		  globals.peakInterval = $2;
		  $$ = 0;
		}
	| CFG_IV_DAILY INTEGER
		{
		  globals.dailyInterval = $2;
		  $$ = 0;
		}
	| TABLE_SIZE INTEGER
		{
		  if ( $1 < Global::S_END )
		    globals.tableSize[$1] = $2;
		  else
		    fputs( "illegal table size marker\n", stderr );
		  $$ = 0;
		}
	| CFG_DNS_POS_TTL INTEGER
		{
		  globals.dnsPositiveTTL = $2;
		  $$ = 0;
		}
	| CFG_DNS_NEG_TTL INTEGER
		{
		  globals.dnsNegativeTTL = $2;
		  $$ = 0;
		}
	| CFG_DNS_FILE QSTRING
		{
		  free((void*) globals.dnsCacheFile );
		  globals.dnsCacheFile = strdup($2);
		  free((void*) $2);
		  $$ = 0;
		}
	| CFG_DNS_CHILDREN INTEGER
		{
		  globals.setChildren($2,globals.dnsChildren);
		  $$ = 0;
		}
	| CFG_DNS_HELPER QSTRING
		{
		  free((void*) globals.dnsHelper );
		  globals.dnsHelper = $2;
		  $$ = 0;
		}
	| CFG_IRR_CHILDREN INTEGER
		{
		  globals.setChildren($2,globals.irrChildren);
		  $$ = 0;
		}
	| CFG_IRR_HELPER QSTRING
		{
		  free((void*) globals.irrHelper );
		  globals.irrHelper = $2;
		  $$ = 0;
		}
	| CFG_IRR_POS_TTL INTEGER
		{
		  globals.irrPositiveTTL = $2;
		  $$ = 0;
		}
	| CFG_IRR_NEG_TTL INTEGER
		{
		  globals.irrNegativeTTL = $2;
		  $$ = 0;
		}
	| CFG_IRR_FILE QSTRING
		{
		  free((void*) globals.irrCacheFile );
		  globals.irrCacheFile = strdup($2);
		  free((void*) $2);
		  $$ = 0;
		}
	| CFG_IRR_SERVER QSTRING
		{
		  if ( globals.irrServer ) free((void*) globals.irrServer );
		  globals.irrServer = strdup($2);
		  free((void*) $2);
		  $$ = 0;
		}
	| CFG_IRR_SERVER OPT_NONE
		{
		  if ( globals.irrServer ) free((void*) globals.irrServer );
		  globals.irrServer = 0;
		  $$ = 0;
		}
	| CFG_LENGTH_BYBYTE BOOLEAN
		{ 
		  globals.sortDBaseByByte = $2;
		  $$ = 0;
		}
	| CFG_REPORT_IV INTEGER
		{ 
		  globals.progressReports = $2;
		  $$ = 0;
		}
	| CFG_METHOD '{' items
		{ 
		  LogLine::mtable = *$3;
		  $$ = $3->distinct();
		  delete $3;
		}
	| CFG_WARN_METHOD BOOLEAN
		{       
		  globals.warnUnknownMethod = $2;
		  $$ = 0;
		}
	| CFG_HIER '{' hier
		{ 
		  LogLine::htable = *$3;
		  $$ = $3->distinct();
		  delete $3;
		}
	| CFG_WARN_HIER BOOLEAN
		{       
		  globals.warnUnknownHier = $2;
		  $$ = 0;
		}
	| CFG_STATUS '{' items
		{ 
		  LogLine::stable = *$3;
		  $$ = $3->distinct();
		  delete $3;
		}
	| CFG_WARN_STATUS BOOLEAN
		{       
		  globals.warnUnknownStatus = $2;
		  $$ = 0;
		}
	| CFG_WARN_NONE_TCP BOOLEAN
		{       
		  globals.warnNoneAsTCP = $2;
		  $$ = 0;
		}
	| CFG_SCHEME '{' slist
		{
		  LogLine::schemes = *$3;
		  $$ = $3->distinct();
		  delete $3;
		}
	| CFG_WARN_SCHEME BOOLEAN
		{       
		  globals.warnUnknownScheme = $2;
		  $$ = 0;
		}
	| CFG_SUFFIX '{' dlist
		{
		  LogLine::suffices = *$3;
		  $$ = $3->distinct();
		  delete $3;
		}
	| CFG_WARN_SUFFIX BOOLEAN
		{       
		  globals.warnUnknownSuffix = $2;
		  $$ = 0;
		}
	| CFG_MEDIA '{' slist
		{
		  LogLine::mediatypes = *$3;
		  if ( LogLine::subtypes ) 
		    delete[] LogLine::subtypes;
		  LogLine::subtypes = 
		    new LinkedListTrie[ ($$ = $3->distinct()) + 1 ];
		  delete $3;
		}
	| CFG_SUBTYPES QSTRING '{' slist
		{
		  insertSubtype( $2, $4 );
		  $$ = $4->distinct();
		  delete $4;
		  free( (void*) $2 );
		}
	| CFG_DOMAIN '{' dlist
		{
		  LogLine::domains = *$3;
		  $$ = $3->distinct();
		  delete $3;
		}
	| CFG_WARN_TLD BOOLEAN
		{       
		  globals.warnUnknownTLD = $2;
		  $$ = 0;
		}
	;

items	: items item
		{ /* adding item to map */
		  Item* temp = (Item*) $2;
		  insertItem( $1, temp, counter );
		  delete temp;
		}
	| /* empty rule */
		{ /* starting map */
		  counter = 0;
		  $$ = new SymbolTrie32();
		}
	;

hier	: hier item
		{ /* adding item to map */
		  Item* temp = (Item*) $2;
		  insertItem( $1, temp, counter );

		  /* adding TIMEOUT_item to map, too */
		  temp->makeTimeout();
		  insertItem( $1, temp, counter );
		  delete temp;
		}
	| /* empty rule */
		{ /* starting map */
		  counter = 0;
		  $$ = new SymbolTrie32();
		}
	;

delim	: '}'
	| ','
	;

item	: ID options delim
		{ $$ = new Item($1,$2,false); }
	| ID OPT_ALIAS ID delim
		{ $$ = new Item($1,$3,true); }
	;

options	: options option
		{ $$ = $1 | $2; }
	| /* empty rule */
		{ $$ = C_U32(0); }
	;

option	: OPT_HIT
		{ $$ = IS_HIT; }
	| OPT_MISS
		{ $$ = IS_MISS; }
	| OPT_NONE
		{ $$ = IS_NONE; }
      	| OPT_TCP
		{ $$ = IS_TCP; }
	| OPT_UDP
		{ $$ = IS_UDP; }
	| OPT_DIRECT
		{ $$ = IS_DIRECT; }
	| OPT_PARENT
		{ $$ = IS_PARENT; }
	| OPT_PEER
		{ $$ = IS_PEER; }
	;

slist	: slist sitem
		{
		  Item* temp = (Item*) $2;
		  insertItem( $1, temp, counter );
		  delete temp;		  
		}
	| /* empty rule */
		{ // starting map
		  counter = 0;
		  $$ = new LinkedListTrie();
		}
	;


dlist	: dlist sitem
		{
		  Item* temp = (Item*) $2;
		  insertItem( $1, temp, counter );
		  delete temp;		  
		}
	| /* empty rule */
		{ // starting map
		  counter = 0;
		  $$ = new SymbolTrie128();
		}
	;



sitem	: QSTRING delim
		{ $$ = new Item($1,false); }
	| QSTRING OPT_ALIAS QSTRING delim
		{ $$ = new Item($1,$3,true); }
	;
%%

static const char *RCS_ID = 
  "$Id: config.y,v 1.12 2000/07/29 22:13:07 voeckler Exp voeckler $";

bool 
insertSubtype( const char* major, LinkedListTrie* minor )
{
  char msg[256];
  MyUInt32 mediatype = LogLine::mediatypes.find(major) & 0xFFFF;

  if ( mediatype && LogLine::subtypes ) {
    if ( LogLine::subtypes[mediatype].size() ) {
      sprintf( msg, "media type \"%s\" already has subtypes!", major );
      strncat( msg, "I am sorry, but currently\nyou cannot append"
	       "more media subtypes to a media type.\n", sizeof(msg) );
      yyerror(msg);
      return false;
    } else {
      LogLine::subtypes[mediatype] = *minor;
      return true;
    }
  } else {
    // not found, ignore subtype trie
    sprintf( msg, "media type \"%s\" not (yet) defined!", major );
    strncat( msg, "I am sorry, but currently\nyou must defined the"
	     "media type before you can define subtypes for it.\n", 256 );
    yyerror(msg);
    return false;
  }
}

int
intlog10( MyUInt32 x )
{
  int n = 0;
  do {
    x /= 10;
    ++n;
  } while ( x );
  return n;
}

void
prepend( const char* prefix, const char*& sptr )
  // purpose: preprend a suffix in front of a string pointer
  // warning: free()s and reallocates the sptr 
{  
  size_t size = strlen(prefix) + strlen(sptr) + 1;
  char* local = (char*) malloc(size);
  strcpy( local, prefix );
  strcat( local, sptr );
  free((void*) sptr );
  sptr = local;
}
