#!/usr/bin/perl
require 5.005;
use strict;

use File::Basename;
use lib dirname($0);
use Tables;
use DBI;
#$main::DEBUG=15;

my $valid = shift || die "need a final time stamp!\n";

# some global variables, adjust according to your needs:
#
# $dbhost: the FQDN of the host where the database server resides.
# $dbname: the name of the database driver you are using.
# $dbuser: the username of the login account to the database
# $dbpass: the password for the login onto the database
# $dborg: The organization as needed for DBase authentication
my ($dbhost,$dbname,$dbuser,$dbpass,$dborg);
$dbhost = 'hawaii.rvs.uni-hannover.de';
$dbname = 'Oracle';
$dbuser = '';
$dbpass = '';
# This is the alternative way to extract from cmdline
($dbuser,$dbpass) = split(/\//,$ENV{ORACLE_USERID});
($dbpass) = split(/\@/,$dbpass) if ( index($dbpass,'@') != -1 );

$dborg = $ENV{ORACLE_SID} || $ENV{TWO_TASK} || 'RVS';

# connect to your database
print STDERR "# trying to connect to DB\n";
$main::handle = DBI->connect( "dbi:Oracle:$dborg", $dbuser, $dbpass,
			      { RaiseError => 1, AutoCommit => 0 } ) || 
    die "can\'t connect: $DBI::errstr\n";

#
# some error handling in order to roll back mistaken insertions
#
BEGIN {
    $main::skip = -1;
}

$main::skip = 0;
$SIG{INT} = sub { exit(1); };

END {
    $main::handle->rollback() if $main::skip == 0;
}

#
# --------------------------------------------------------------- subs
#

#
# --------------------------------------------------------------- main
#

my ($ref);
print STDERR "# getting all timestamp information from sf_stamp1\n";
$ref = $main::handle->selectall_arrayref( qq{ 
    select i1id from sf_stamp1 where first >= $valid 
    }) || die "sf_stamp1: ", DBI->errstr, "\n";

my @sf_i1 = grep {!/(PEAK|STAMP|META)/} @main::tables;
foreach my $row ( @{$ref} ) {
    my $i1id = $row->[0];
    print STDERR "# deleting $i1id\n";
    foreach my $i ( @sf_i1 ) {
	print STDERR "# $i\n" if $main::DEBUG;
	$main::handle->do( qq{delete from $i where i1id=$i1id} ) ||
	    die "$i: ", DBI->errstr, "\n";
    }
    
    $main::handle->do( qq{delete from sf_stamp1 where i1id=$i1id} ) ||
	die "sf_stamp1: ", DBI->errstr, "\n";
}

undef $ref;
$main::handle->commit() ||
    die "commit: $DBI::errstr\n";

print STDERR "# getting all timestamp information from sf_stamp2\n";
$ref = $main::handle->selectall_arrayref( qq{ 
    select i2id from sf_stamp2 where first >= $valid 
    }) || die "sf_stamp2: ", DBI->errstr, "\n";

foreach my $row ( @{$ref} ) {
    my $i2id = $row->[0];
    print STDERR "# deleting $i2id\n";
    $main::handle->do( qq{delete from sf_peak where i2id=$i2id} ) ||
	die "sf_peak: ", DBI->errstr, "\n";
    $main::handle->do( qq{delete from sf_stamp2 where i2id=$i2id} ) ||
	die "sf_stamp2: ", DBI->errstr, "\n";
}

$main::handle->commit() ||
    die "commit: $DBI::errstr\n";
$main::handle->disconnect() ||
    die "disconnect: $DBI::errstr\n";
$main::skip = 1;
print "SUCCESS!\n";
