//
// $Id: logline.hh,v 1.2 2000/07/27 07:13:15 voeckler Exp $
//
// Author:  Jens-S. Vckler <voeckler@rvs.uni-hannover.de>
//
// File:    logline.hh
//          Sun Jul 18 1999
//
// (c) 1998,99 Lehrgebiet Rechnernetze und Verteilte Systeme
//             Universitt Hannover, Germany
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that (i) the above copyright notices and this permission
// notice appear in all copies of the software and related documentation,
// and (ii) the names of the Lehrgebiet Rechnernetze und Verteilte
// Systeme and the University of Hannover may not be used in any
// advertising or publicity relating to the software without the
// specific, prior written permission of Lehrgebiet Rechnernetze und
// Verteilte Systeme and the University of Hannover.
//
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE LEHRGEBIET RECHNERNETZE UND VERTEILTE SYSTEME OR
// THE UNIVERSITY OF HANNOVER BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
// ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
// SOFTWARE.
//
// $Log: logline.hh,v $
// Revision 1.2  2000/07/27 07:13:15  voeckler
// added trie for file extensions (suffix).
//
// Revision 1.1  1999/08/05 21:15:59  voeckler
// Initial revision
//
//
#ifndef _LOGLINE_HH
#define _LOGLINE_HH

#if defined(__GNUC__) || defined(__GNUG__)
#pragma interface
#endif

#include <stdio.h> // FILE*
#include <string.h>
#include "typedefs.h"
#include "input.hh"
#include "lltrie.hh"
#include "symtrie32.hh"
#include "symtrie128.hh"

struct LogLine {
public:
  inline LogLine()
    { clear(); }
  inline ~LogLine()
    { }

  inline void clear()
    // purpose: zero everything within - this is the slow variant
    { memset( &_stamp, 0, &_end - ((char*)((void*) &_stamp)) ); }
  inline void shallow_clear()
    // purpose: make most things appear empty - the fast variant
    { _stamp = _size = _duration = _statuscode = 0; 
      _mediatype = _mediasubtype = _hierarchy = _method = _status = C_U32(0);
      _timeout = false;
      *_requester = *_url = *_ident = *_hierhost = 0; 
    }

  //
  // --- Flags and Tables --------------------------------------------
  //
  static SymbolTrie32 mtable;
  static SymbolTrie32 htable;
  static SymbolTrie32 stable;
  static LinkedListTrie  schemes;
  static LinkedListTrie  mediatypes;
  static LinkedListTrie* subtypes;
  static SymbolTrie128 domains;
  static SymbolTrie128 suffices;

  int parseStatus( BaseInput* in );
  // purpose: read status column and split into message and HTTP reply
  // paramtr: in (IO): pointer to input stream to read from
  // returns: number of characters read, may return 0 to signal an error.
  // warning: changes instance variables _status and _statuscode,
  // warning: _statuscode will be -1, if no dash was found in string

  int parseMethod( BaseInput* in );
  // purpose: parse the 6th column of the squid input file, the HTTP method.
  // paramtr: in (IO): pointer to input stream to get chars from
  // returns: number of characters read
  // warning: changes instance variable _method, check for ERROR

  int parseHierarchy( BaseInput* in );
  // purpose: obtain hierarchy information and split into three parts
  // paramtr: in (IO): stream to read from
  // returns: number of characters read, may return 0 to signal an error.
  // warning: changes i'vars _hierarchy, _timeout and _hierhost,
  // warning: _hierhost will be empty, if no dash was found in string

  int parseMediaType( BaseInput* in );
  // purpose: obtain media type information
  // paramtr: in (IO): stream to read from
  // returns: number of characters read, may return 0 to signal an error.
  // warning: changes i'var _mediatype

  void dump( FILE* out, int amount = 10, const char* prefix = "+" );
  // purpose: dump contents of instance variables
  // paramtr: out (IO): stream to dump contents onto
  //          amount (IN): number of columns which were correctly parsed
  //          prefix (IN): prepend each new line with this message
  // returns: -

  MyUInt32    _stamp;		// e.g. 930175218 w/o the fractional part
  MySInt32    _duration;	// e.g. 35536
  char        _requester[128];	// e.g. "130.75.3.218" or "some.host.here"
  MyUInt32    _status;		// tokenized! e.g. TCP_MEM_HIT
  MySInt32    _statuscode;	// e.g. 200
  MyUInt64    _size;		// e.g. 4137
  MyUInt32    _method;		// tokenized! e.g. GET
  char        _url[16384];	// e.g. "http://www.uni-frankfurt.de/"
  char        _ident[16];	// e.g. "-"
  bool        _timeout;		// prefix from TIMEOUT_xxxx hierarchy tag
  MyUInt32    _hierarchy;	// tokenized! e.g. DIRECT/a.b.c.d
  char        _hierhost[128];	// e.g. "leipzig.www-cache.dfn.de"
  MyUInt32    _mediatype;	// tokenized! e.g. text, or image
  MyUInt32    _mediasubtype;	// tokenized! e.g. "html" or "octet-stream".
  char        _end;
};

#endif // _LOGLINE_HH
